#include <NiftyTitleBar.h>
#include <GNUstepGUI/GSDisplayServer.h>
#include <AppKit/PSOperators.h>

#define TITLE_HEIGHT 23.0
#define RESIZE_HEIGHT 9.0
static NSColor *titleColor[3];
static NSDictionary *titleTextAttributes[3];

extern id<GSWindowDecorator> windowDecorator;

@class GSBackendWindowDecorationView;

@implementation GSWindowDecorationView (NiftyTitleBar)

+(id<GSWindowDecorator>)windowDecorator
{
  if ([GSCurrentServer() handlesWindowDecorations])
    return [GSBackendWindowDecorationView self];
  else
  return [NiftyWindowDecorationView self];
}
@end

@implementation NiftyWindowDecorationView : GSStandardWindowDecorationView

- initWithFrame:(NSRect)frame
         window:(NSWindow*)w
{
  self = [super initWithFrame:frame window:w];
  if (!titleTextAttributes[0])
    {
      titleTextAttributes[0] = [[NSMutableDictionary alloc]
        initWithObjectsAndKeys:
          [NSFont titleBarFontOfSize: 0], NSFontAttributeName,
          [NSColor windowFrameTextColor], NSForegroundColorAttributeName,
          nil];
      titleTextAttributes[1] = [[NSMutableDictionary alloc]
        initWithObjectsAndKeys:
          [NSFont titleBarFontOfSize: 0], NSFontAttributeName,
          [NSColor blackColor], NSForegroundColorAttributeName, /* TODO: need a
named color for this */
          nil];
      titleTextAttributes[2] = [[NSMutableDictionary alloc]
        initWithObjectsAndKeys:
          [NSFont titleBarFontOfSize: 0], NSFontAttributeName,
          [NSColor windowFrameTextColor], NSForegroundColorAttributeName,
          nil];

      titleColor[0] = RETAIN([NSColor windowFrameColor]);
      titleColor[1] = RETAIN([NSColor lightGrayColor]);
      titleColor[2] = RETAIN([NSColor darkGrayColor]);
    } 
  if ([w representedFilename] && [[w representedFilename] length] && ![[w representedFilename] isEqual: @"Window"])
    {
      NSImage *icon;
      hasAvatar = YES; 
      avatar = [[TitleBarFileIcon alloc] init];
      icon = [[[NSWorkspace sharedWorkspace] iconForFile:[w representedFilename]] copy];
      [icon setScalesWhenResized:YES];
      [icon setSize:NSMakeSize(TITLE_HEIGHT - 5, TITLE_HEIGHT - 5)];
      [avatar setBordered:NO];
      [avatar setImage:icon];
      RELEASE(icon); 
      [avatar setImagePosition:NSImageOnly];
      [self addSubview:avatar];
      [self updateRects];
    }
  return self;
}
- (void) drawTitleBar
{
static const NSRectEdge edges[4] = {NSMinXEdge, NSMaxYEdge,
                                    NSMaxXEdge, NSMinYEdge};
  float grays[3][4] =
    {{NSLightGray, NSLightGray, NSDarkGray, NSDarkGray},
    {NSWhite, NSWhite, NSDarkGray, NSDarkGray},
    {NSLightGray, NSLightGray, NSBlack, NSBlack}};
  NSRect workRect;
  NSString *title = [window title];
  /*
  Draw the black border towards the rest of the window. (The outer black
  border is drawn in -drawRect: since it might be drawn even if we don't have
  a title bar.
  */
  [[NSColor blackColor] set];
  PSmoveto(0, NSMinY(titleBarRect) + 0.5);
  PSrlineto(titleBarRect.size.width, 0);
  PSstroke();

  /*
  Draw the button-like border.
  */
  workRect = titleBarRect;
  workRect.origin.x += 1;
  workRect.origin.y += 1;
  workRect.size.width -= 2;
  workRect.size.height -= 2;

  workRect = NSDrawTiledRects(workRect, workRect, edges, grays[inputState], 4);

  /*
  Draw the background.
  */
  [titleColor[inputState] set];
  NSRectFill(workRect);

  /* Draw the title. */
  if (isTitled)
    {
      NSSize titleSize;

      if (hasMiniaturizeButton)
        {
          workRect.origin.x += 17;
          workRect.size.width -= 17;
        }
      if (hasCloseButton)
        {
          workRect.size.width -= 17;
        }

      titleSize = [title sizeWithAttributes: titleTextAttributes[inputState]];
      if (titleSize.width <= workRect.size.width)
        workRect.origin.x += NSMidX(workRect) - titleSize.width / 2;
      workRect.origin.y = NSMidY(workRect) - titleSize.height / 2;
      workRect.size.height = titleSize.height;
      [title drawInRect: workRect
         withAttributes: titleTextAttributes[inputState]];

      if (hasAvatar)
        {
          [avatar setFrame: NSMakeRect(workRect.origin.x - TITLE_HEIGHT - 2,
                                       titleBarRect.origin.y+2,
                                       TITLE_HEIGHT - 5,
                                       TITLE_HEIGHT - 5)];
        }
    }
}

-(void) setRepresentedFilename:(NSString*)path
{
  if (path && [path length] && ![path isEqual: @"Window"] )
    {
      NSImage *icon;
      hasAvatar = YES;
      avatar = [[TitleBarFileIcon alloc] init];
      icon = [[[NSWorkspace sharedWorkspace] iconForFile:path] copy];
      [icon setScalesWhenResized:YES];
      [icon setSize:NSMakeSize(TITLE_HEIGHT - 5, TITLE_HEIGHT - 5)];
      [avatar setBordered:NO];
      [avatar setEnabled:NO];
      [avatar setImage:icon];
      RELEASE(icon); 
      [avatar setImagePosition:NSImageOnly];
      [avatar setHighlightsBy:NSChangeBackgroundCellMask];
      [self addSubview:avatar];
      RELEASE(avatar);
      [self setNeedsDisplayInRect:titleBarRect];
    }
}

@end

@implementation NSWindow (UpdateTitleBar)
- (void) setRepresentedFilename: (NSString*)aString
 {
   ASSIGN(_representedFilename, aString);
   if ([_wv respondsToSelector:@selector(setRepresentedFilename:)])
     [(NiftyWindowDecorationView*)_wv setRepresentedFilename:aString];
 }

@end
